from aiohttp import web
from keybert import KeyBERT
import json
import os

app = web.Application()
models = {}


# Ping
async def ping(request):
    return web.json_response({'status': True, 'message': None, 'data': True})


# Load a model
async def load_model(request):
    try:
        data = await request.json()
        model_name = data.get('model_name', '')
        if model_name:
            models[model_name] = KeyBERT(model=model_name)
            return web.json_response({'status': True, 'message': None, 'data': f'Model {model_name} loaded.'})
        else:
            return web.json_response({'status': False, 'message': 'No model name provided.', 'data': None})
    except Exception as e:
        return web.json_response({'status': False, 'message': str(e), 'data': None})


# Extract keywords
async def extract_keywords(request):
    try:
        data = await request.json()
        model_name = data.get('model_name', '')
        docs = data.get('docs', [])
        candidates = data.get('candidates', None)
        keyphrase_ngram_range = tuple(data.get('keyphrase_ngram_range', (1, 1)))
        stop_words = data.get('stop_words', 'english')
        top_n = data.get('top_n', 5)
        min_df = data.get('min_df', 1)
        use_maxsum = data.get('use_maxsum', False)
        use_mmr = data.get('use_mmr', False)
        diversity = data.get('diversity', 0.5)
        nr_candidates = data.get('nr_candidates', 20)
        seed_keywords = data.get('seed_keywords', None)

        # Check if the model is loaded, if not, load it
        if model_name not in models:
            models[model_name] = KeyBERT(model=model_name)

        if docs:
            keywords = models[model_name].extract_keywords(
                docs,
                candidates=candidates,
                keyphrase_ngram_range=keyphrase_ngram_range,
                stop_words=stop_words,
                top_n=top_n,
                min_df=min_df,
                use_maxsum=use_maxsum,
                use_mmr=use_mmr,
                diversity=diversity,
                nr_candidates=nr_candidates,
                seed_keywords=seed_keywords,
            )
            # Transform the result from a list of tuples to a dictionary
            keywords_dict = {word: score for word, score in keywords}
            return web.json_response({'status': True, 'message': None, 'data': keywords_dict})
        else:
            return web.json_response(
                {'status': False, 'message': 'No document provided.', 'data': None})
    except Exception as e:
        return web.json_response({'status': False, 'message': str(e), 'data': None})


if __name__ == '__main__':
    # Check for required environment variables
    if 'KEYBERT_ENABLED' not in os.environ or os.environ['KEYBERT_ENABLED'] != '1':
        print('Error: KEYBERT_ENABLED environment variable not set or not "true".')
        exit(1)

    if 'KEYBERT_PORT' not in os.environ:
        print('Error: KEYBERT_PORT environment variable not set.')
        exit(1)

    # Preload models if KEYBERT_PRELOAD is set
    if 'KEYBERT_PRELOAD' in os.environ:
        preload_models = json.loads(os.environ['KEYBERT_PRELOAD'])
        for model_name in preload_models:
            models[model_name] = KeyBERT(model=model_name)
            print(f'Model {model_name} preloaded.')

    # Add routes
    app.router.add_post('/ping', ping)
    app.router.add_post('/load_model', load_model)
    app.router.add_post('/extract_keywords', extract_keywords)

    # Start server
    web.run_app(app, host='0.0.0.0', port=int(os.environ['KEYBERT_PORT']))